# message_parser.py
#
# Copyright 2024 Christopher Talbot
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <https://www.gnu.org/licenses/>.
#
# SPDX-License-Identifier: GPL-3.0-or-later

from gi.repository import Adw
from gi.repository import Gtk
import gettext
import meshtastic.serial_interface
import meshtastic.mesh_interface
from pubsub import pub
import json
import time
import os

import gtk_meshtastic_client.utils as utils

class MessageParser():

    def onReceive(self, packet, interface):
        app = Gtk.Application.get_default()
        if app.app_settings.get_log_packets():
            json_packet = json.dumps(packet, indent=4, default=lambda s: " ".join(str(s).split()))
            if os.path.exists(utils.messages_file_path):
                with open(utils.messages_file_path, 'r+') as json_file:
                    content = json_file.read().strip()
                    if content:
                        if content.endswith(']'):
                            content = content[:-1].rstrip() + ',\n'
                        else:
                            content = '['
                        json_file.seek(0)
                        json_file.write(content + json_packet + '\n]')
                    else:
                        json_file.seek(0)
                        json_file.write('[' + json_packet + '\n]')
                    json_file.truncate()
            else:
                with open(utils.messages_file_path, 'w') as json_file:
                    json_file.write('[\n' + json_packet + '\n]')

        if app.app_settings.get_print_packets():
            print(f"{packet} \n\n")

        if 'decoded' in packet:
            app = Gtk.Application.get_default()
            win = Gtk.Application.get_active_window(app)

            if packet['decoded'].get('portnum') == 'TELEMETRY_APP':
                win.nearby_nodes_page_bin.packet_node_update(packet, interface)
                return
            elif packet['decoded'].get('portnum') == 'NODEINFO_APP':
                win.nearby_nodes_page_bin.packet_node_update(packet, interface)
                return
            elif packet['decoded'].get('portnum') == 'POSITION_APP':
                win.nearby_nodes_page_bin.packet_node_update(packet, interface)
                return

            elif packet['decoded'].get('portnum') == 'TEXT_MESSAGE_APP':
                win.channels_page_bin.new_text_message(packet, interface)
                return

            elif packet['decoded'].get('portnum') == 'ROUTING_APP':
                if 'priority' in packet:
                    if packet['priority'] == "ACK":
                         win.channels_page_bin.process_ack_message(packet, interface)
                         return
                if packet['to'] == interface.myInfo.my_node_num:
                    win.channels_page_bin.process_routing_app_message(packet, interface)
                    return

    def __init__(self):
        # The python library will authmatically process any messages stored, so there
        # is no need to manually scan for messages on connection
        pub.subscribe(self.onReceive, 'meshtastic.receive')
